function [ dmsDataStruct, ridges, NumClusters_pos, NumClusters_neg ] = detectRidges( dmsDataStruct, polyOrder, detectRidgeSettings )
%detectRidges finds ridges in dms dispersion plot and uses a polynomial
%equation to fit a line to the ridges.
%
% Input:        - dmsDataStruct
%               - order of the polynomial. options include: 'poly1', 'poly2', or 'poly3'
% Ouput:        - updated dmsDataStruct to include objects segmented by
% function
%               - ridges is struct containing list of all ridges/splines found from all
%               samples in positive spectrum and negative spectrum

%
%   Author: Paul Hichwa
%   Date written/updated: 25sep2017

%% Initialize outputs:
% Initialize ridges struct
ridges = struct('pos', [],...
                'neg', []);
            
%% Initialize detection settings
segThresh = detectRidgeSettings.segmentationThresh;
minimumPixels = detectRidgeSettings.minPixels;

% Initialize array for size of allFeatureVectors to determine NumClusters
sizes_pos = zeros(1,size(dmsDataStruct,2));
sizes_neg = zeros(1,size(dmsDataStruct,2));

%% Loop through dmsDataStruct and update it with objects found
for i = 1:size(dmsDataStruct,2)

    %% Check if there is data in the dispersion image for the positive data
    if isempty(dmsDataStruct(i).dispersion_pos)
        dmsDataStruct(i).ridgeObjects_pos = [];
        dmsDataStruct(i).bwfinal_pos = [];
    else
        %% Detect objects in positive spectra
        positiveSpectrum = dmsDataStruct(i).dispersion_pos;
        [dmsDataStruct(i).ridgeObjects_pos, dmsDataStruct(i).bwfinal_pos, dmsDataStruct(i).xplot_pos, dmsDataStruct(i).yplot_pos] = objectDetectionFunc(positiveSpectrum, polyOrder, segThresh, minimumPixels);
        
    end

    %% Check if there is data in dispersion image for negative data
    if ~isempty(dmsDataStruct(i).dispersion_neg)
        %% Detect spline objects (NEED TO UPDATE)
        negativeSpectrum = dmsDataStruct(i).dispersion_neg;
        [dmsDataStruct(i).ridgeObjects_neg, dmsDataStruct(i).bwfinal_neg, dmsDataStruct(i).xplot_neg, dmsDataStruct(i).yplot_neg] = objectDetectionFunc(negativeSpectrum, polyOrder, segThresh, minimumPixels);
        
    else
        dmsDataStruct(i).ridgeObjects_neg = [];
        dmsDataStruct(i).bwfinal_neg = [];
    end
    
    % Concatenate all ridge object feature vectors from all pos and neg
    % dispersion images respectively. Result is MxN matrix where M (rows)
    % is the number of feature vectors and N (columns) is the number of
    % descriptors for each feature vector
    ridges.pos = [ridges.pos; dmsDataStruct(i).ridgeObjects_pos];
    ridges.neg = [ridges.neg; dmsDataStruct(i).ridgeObjects_neg];
    
    % Update size array for determining number of clusters to use
    sizes_pos(1,i) = size(dmsDataStruct(i).ridgeObjects_pos, 1);
    sizes_neg(1,i) = size(dmsDataStruct(i).ridgeObjects_neg, 1);
        
    
end

% number of cluster for kmeans clustering used in generate codebook
% function
NumClusters_pos = max(sizes_pos);
NumClusters_neg = max(sizes_neg);


end


%% Local function for detecting spline objects
% Update this if find a better method for segmenting and detecting spline
% objects
function [ ridgeFeatures, BWfinal, xplotPolyPoints, yplotPolyPoints ] = objectDetectionFunc(dispersion, orderOfPoly, segmentationThresh, minimumPixels)

%% initialize polyPoints
xplotPolyPoints = [];
yplotPolyPoints = [];

%% phase symmetry and phase congruency calculation
phaseSymImage = phasesym(dispersion);

% phase congruency calculation
PC = phasecongmono(phaseSymImage);
    
%% segmentation using Otsu method (i.e. global threshold)
% % level = graythresh(PC);     % old versio
level = segmentationThresh;
BW = im2bw(PC,level);

%% thining morphological operation
bwThin = bwmorph(BW, 'thin', Inf);

%% cleaning up stray pixels
bwThinCleaned = bwareaopen(bwThin, minimumPixels);       % NOTE: can modify 2nd input to adjust. default 10

%% clean up bottom 3 rows so no connectivity of curves off the wall.
bwThinCleaned(end,:) = 0;
bwThinCleaned(end-1,:) = 0;
bwThinCleaned(end-2,:) = 0;
bwThinCleaned(end-3,:) = 0;
% % bwThinCleaned(end-4,:) = 0;
% % bwThinCleaned(end-5,:) = 0;

%% Separate ridge objects from eachother
cc = bwconncomp(bwThinCleaned);
BWfinal = bwThinCleaned;

%% set up ridgeFeatures and gof vectors
if strcmp(orderOfPoly, 'poly1')
    ridgeFeatures = zeros(cc.NumObjects,12);
elseif strcmp(orderOfPoly,'poly2')
    ridgeFeatures = zeros(cc.NumObjects,13);
elseif strcmp(orderOfPoly, 'poly3')
    ridgeFeatures = zeros(cc.NumObjects, 14);
elseif strcmp(orderOfPoly, 'AutoDetermine')
    ridgeFeatures = zeros(cc.NumObjects, 14);
end
    % % evaluateLineFit = zeros(cc.NumObjects,1);       % might not need this, but there just in case

%% Loop through all curves for a sample
for k = 1:cc.NumObjects
    curveOfInterest = ismember(labelmatrix(cc), k);
 
    % Find points on curve of interest
    [ys, xs] = find(curveOfInterest);
    
    % Rotate right by 90 degrees so there is only 1 y for every x. The
    % branches split from up to right
    x = ys;
    y = xs;
    
    % Check to see if there are at least 4 data points to fit the model
    if length(x) < 4
        continue
    end
    
    % Testing polynomial fit for description of the ridges
    [xpolyData, ypolyData] = prepareCurveData( x, y );
    
    % Check to see if auto determination for polynomial order is chosen
    if strcmp(orderOfPoly, 'AutoDetermine')
        
        % Generate all order of polynomial
        ftPoly1 = fittype('poly1');
        ftPoly2 = fittype('poly2');
        ftPoly3 = fittype('poly3');
        
        % fit all models to data
        [fitresult1, gof1] = fit(xpolyData, ypolyData, ftPoly1);
        [fitresult2, gof2] = fit(xpolyData, ypolyData, ftPoly2);
        [fitresult3, gof3] = fit(xpolyData, ypolyData, ftPoly3);
        
        % Check the R for all polynomials
        gofTot = [gof1.rsquare, gof2.rsquare, gof3.rsquare];
        [maxR,indexM] = max(gofTot);
        
        if indexM == 1
            yfitDataPoly = fitresult1(xpolyData);
            polyOrderChosen = 1;
        elseif indexM == 2
            yfitDataPoly = fitresult2(xpolyData);
            polyOrderChosen = 2;
        elseif indexM == 3
            yfitDataPoly = fitresult3(xpolyData);
            polyOrderChosen = 3;
        end
        
    else
        % Set up fittype and options.
        ftPoly = fittype( orderOfPoly );
        
        % Fit model to data.
        [fitresult_poly, gof_poly] = fit( xpolyData, ypolyData, ftPoly );
        
        % Get interpreted data points
        yfitDataPoly = fitresult_poly(xpolyData);
    end
    
    % Calculate interpreted data points
    xplotPolyPoints = [yfitDataPoly; xplotPolyPoints];
    yplotPolyPoints = [xpolyData; yplotPolyPoints];
        
    % CurveofInterest only has one curve. regionprops uses 8 neighborhood
    % to look at the connected component to extract certain properties of
    % the streak. the boundingbox gives the bounding box of streak so you
    % can extrapolate just the streak
    % Extract data from smoothing splines (use coefficients for clustersing??)
    objectDescriptors = regionprops(curveOfInterest, 'Area', 'Centroid',...
                                    'Eccentricity', 'Perimeter',...
                                    'Orientation', 'BoundingBox');
    
    % normalize the location data over the length and width of dispersion plot
    xCentroid = objectDescriptors.Centroid(1) / size(dispersion,2);
    yCentroid = objectDescriptors.Centroid(2) / size(dispersion,1);
    xUpperLeftCorner = objectDescriptors.BoundingBox(1) / size(dispersion,2);
    yUpperLeftCorner = objectDescriptors.BoundingBox(2) / size(dispersion,1);
    xWidthBB = objectDescriptors.BoundingBox(3) / size(dispersion,2);
    yWidthBB = objectDescriptors.BoundingBox(4) / size(dispersion,1);
    
    %% Parse order of the polynomial and create feature vector for riges
    switch orderOfPoly
        case 'poly1'
            ridgeFeatures(k,:) = [fitresult_poly.p1, fitresult_poly.p2,...
                                  objectDescriptors.Area, xCentroid,...
                                  yCentroid, objectDescriptors.Eccentricity,...
                                  objectDescriptors.Perimeter, objectDescriptors.Orientation,...
                                  xUpperLeftCorner, yUpperLeftCorner,...
                                  xWidthBB, yWidthBB];
        case 'poly2'
            ridgeFeatures(k,:) = [fitresult_poly.p1, fitresult_poly.p2, fitresult_poly.p3,...
                                  objectDescriptors.Area, xCentroid,...
                                  yCentroid, objectDescriptors.Eccentricity,...
                                  objectDescriptors.Perimeter, objectDescriptors.Orientation,...
                                  xUpperLeftCorner, yUpperLeftCorner,...
                                  xWidthBB, yWidthBB];
        case 'poly3'
            ridgeFeatures(k,:) = [fitresult_poly.p1, fitresult_poly.p2, fitresult_poly.p3, fitresult_poly.p4...
                                  objectDescriptors.Area, xCentroid,...
                                  yCentroid, objectDescriptors.Eccentricity,...
                                  objectDescriptors.Perimeter, objectDescriptors.Orientation,...
                                  xUpperLeftCorner, yUpperLeftCorner,...
                                  xWidthBB, yWidthBB];
        case 'AutoDetermine'
            
            % check to see which polynomial order was the best
            if polyOrderChosen == 1
                ridgeFeatures(k,:) = [fitresult1.p1, fitresult1.p2,0,0....
                                      objectDescriptors.Area, xCentroid,...
                                      yCentroid, objectDescriptors.Eccentricity,...
                                      objectDescriptors.Perimeter, objectDescriptors.Orientation,...
                                      xUpperLeftCorner, yUpperLeftCorner,...
                                      xWidthBB, yWidthBB];
            elseif polyOrderChosen == 2
                ridgeFeatures(k,:) = [fitresult2.p1, fitresult2.p2, fitresult2.p3,0,...
                                      objectDescriptors.Area, xCentroid,...
                                      yCentroid, objectDescriptors.Eccentricity,...
                                      objectDescriptors.Perimeter, objectDescriptors.Orientation,...
                                      xUpperLeftCorner, yUpperLeftCorner,...
                                      xWidthBB, yWidthBB];            
            elseif polyOrderChosen == 3
                ridgeFeatures(k,:) = [fitresult3.p1, fitresult3.p2, fitresult3.p3, fitresult3.p4...
                                      objectDescriptors.Area, xCentroid,...
                                      yCentroid, objectDescriptors.Eccentricity,...
                                      objectDescriptors.Perimeter, objectDescriptors.Orientation,...
                                      xUpperLeftCorner, yUpperLeftCorner,...
                                      xWidthBB, yWidthBB];
            end
            
            
    end
    
end


end

% AnalyzeIMS is the proprietary property of The Regents of the University
% of California (The Regents.) 
% 
% Copyright  2014-20 The Regents of the University of California, Davis
% campus. All Rights Reserved. 
%
% This material is available as open source for research and personal use 
% under a PolyForm Noncommercial License 1.0.0 
% (https://polyformproject.org/licenses/noncommercial/1.0.0/). 
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted by nonprofit, research institutions for
% research use only, provided that the following conditions are met:  
% 
% - Redistributions of source code must retain the above copyright notice,
% this list of conditions and the following disclaimer. 
% 
% - Redistributions in binary form must reproduce the above copyright
% notice, this list of conditions and the following disclaimer in the
% documentation and/or other materials provided with the distribution.  
% 
% - The name of The Regents may not be used to endorse or promote products
% derived from this software without specific prior written permission. 
% 
% The end-user understands that the program was developed for research
% purposes and is advised not to rely exclusively on the program for any
% reason.  
% 
% THE SOFTWARE PROVIDED IS ON AN "AS IS" BASIS, AND THE REGENTS HAS NO
% OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
% MODIFICATIONS. THE REGENTS SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED
% WARRANTIES, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
% NO EVENT SHALL THE REGENTS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
% SPECIAL, INCIDENTAL, EXEMPLARY OR CONSEQUENTIAL DAMAGES, INCLUDING BUT
% NOT LIMITED TO PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, LOSS OF USE,
% DATA OR PROFITS, OR BUSINESS INTERRUPTION, HOWEVER CAUSED AND UNDER ANY
% THEORY OF LIABILITY WHETHER IN CONTRACT, STRICT LIABILITY OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
% THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF ADVISED OF THE POSSIBILITY
% OF SUCH DAMAGE.            
% 
% If you do not agree to these terms, do not download or use the software.
% This license may be modified only in a writing signed by authorized
% signatory of both parties.
% 
% For commercial license information please contact copyright@ucdavis.edu.
